""" Unit tests for core_linked_records_app.utils.dict
"""
from unittest import TestCase


class TestGetDictValueFromKeyList(TestCase):
    def test_empty_key_list_returns_dictionary(self):
        pass

    def test_non_existing_key_returns_none(self):
        pass

    def test_existing_key_returns_dict_content(self):
        pass
