""" Unit tests for core_linked_records_app.utils.blob
"""
from unittest import TestCase


class TestGetBlobDownloadRegex(TestCase):
    def test_no_pid_in_document_returns_empty_list(self):
        pass

    def test_returned_list_has_correct_length(self):
        pass
